#include "lua.h"
#include "lauxlib.h"
#include "tcp.h"


static int lua_tcp_open(lua_State *L)
{
  int result;
  const char *host = luaL_check_string(L, 1);
  const char *port = luaL_check_string(L, 2);
  int server = luaL_opt_int(L, 3, 0);
  
  result = tcp_open(host, port, server);

  if(result < 0)
    lua_pushnil(L);
  else
    lua_pushnumber(L, result); /* socket descriptor */

  return 1;
}

static int lua_tcp_check(lua_State *L)
{
  int result;
  int sd = luaL_check_int(L, 1);
  int sleeptime = luaL_opt_int(L, 2, 0);

  result = tcp_check(sd, sleeptime);

  if(result < 0)
    lua_pushnil(L);
  else
    lua_pushnumber(L, 1); /* ok */

  return 1;
}

/* this call blocks */
static int lua_tcp_accept(lua_State *L)
{
  int result;
  int sd = luaL_check_int(L, 1);
  char *host = NULL;

  result = tcp_accept(sd, &host);

  if(result < 0)
  {
    lua_pushnil(L);
    lua_pushnil(L);
  }
  else
  {
    lua_pushnumber(L, result); /* socket descriptor */
    lua_pushstring(L, host); /* socket hostname */
  }

  return 2;
}

static int lua_tcp_send(lua_State *L)
{
  int result;
  int sd = luaL_check_int(L, 1);
  const char *data = luaL_check_string(L, 2);

  result = tcp_send(sd, data, strlen(data));

  if(result < 0)
    lua_pushnil(L);
  else
    lua_pushnumber(L, result); /* bytes sent */

  return 1;
}

static int lua_tcp_recv(lua_State *L)
{
  int result;
  int sd = luaL_check_int(L, 1);
  char *data;

  result = tcp_recv(sd, &data);

  if(result < 0)
  {
    lua_pushnil(L);
    lua_pushnil(L);
  }
  else
  {
    lua_pushnumber(L, result); /* bytes received */
    lua_pushstring(L, data); /* data received */
  }

  return 2;
}

static int lua_tcp_close(lua_State *L)
{
  int result;
  int sd = luaL_check_int(L, 1);

  result = tcp_close(sd);

  if(result < 0)
    lua_pushnil(L);
  else
    lua_pushnumber(L, 1); /* ok */

  return 1;
}


static const struct luaL_reg tcplib[] =
{
  {"tcp_open", lua_tcp_open},
  {"tcp_check", lua_tcp_check},
  {"tcp_accept", lua_tcp_accept},
  {"tcp_send", lua_tcp_send},
  {"tcp_recv", lua_tcp_recv},
  {"tcp_close", lua_tcp_close}
};

void lua_tcplibopen(lua_State *L)
{
  luaL_openl(L, tcplib);
}
